#!/usr/bin/python
# -*- coding: utf-8 -*-

############################################################################
#    Copyright (C) 2007 by Romain Henriet                                  #
#    henriet.romain@libertysurf.fr                                         #
#                                                                          #
#    DesktopBasket is free software; you can redistribute it and/or modify #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    DesktopBasket is distributed in the hope that it will be useful,      #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

import os, karamba, sys
from xml.dom.minidom import parse
from HTMLParser import HTMLParser
from gettext import GNUTranslations
from StringIO import StringIO
from kdecore import *
from kdeui import KPassivePopup
from commands import getoutput
import locale, time, re
from qt import QPixmap
from desktopbasket_lib import *
from desktopbasket_struc import *

locale.setlocale(locale.LC_ALL, '')

# reads baskets and notes content
notesGatherer = None
# currently displayed post-it
currentPostIt = 0
# all post-its
myPostIts = []
# to force refresh when theme is being maximized and was minimized
# for too long
skippedOnce = False
# counter for refreshing notes
cmpt_refresh=2

# configuration of the theme
myConfig = None
# display elements like buttons, images, ...
dispElmt = None
# actions triggered by the karamba.executeInteractive command
myActions = None

# note for which the highlight is being changed
hlCurrentNote=None

# Translation function
_ = lambda(x): x

# HTML Parser to read notes content
class MyParser(HTMLParser):
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.reset()
	
	def handle_starttag(self, tag, attrs):
		if tag == 'body':
			self.start = self.getpos()

	def handle_endtag(self, tag):
		if tag == 'body':
			self.end=self.getpos()
			self.end = (self.end[0], self.end[1]+7)

	def getPositions(self):
		return self.start, self.end

	def reset(self):
		HTMLParser.reset(self)
		self.content = ''
		self.start = None
		self.end = None


# Searches for baskets, parses their contents and 
# reads notes
class NotesGatherer:

	def __init__(self):
		self.base = myConfig.basketBase
		self.lastTagsChange = None
		self.choosingTags = {}
		self.lastBasketListChange = None
		self.basketsList=[]
		#self.basketsChanges=[]
		self.needUpdate = True
		self.desiredTag = myConfig.desiredTag
		self.reloadTags()
		
	def main(self):
		#self.needUpdate = False
		self.notesList=[]
		self.myHTMLParser = MyParser()
		self.getBaskets()
		self.reloadTags()
		if self.desiredTag!=myConfig.desiredTag:
			self.needUpdate=True
			self.desiredTag=myConfig.desiredTag
		if not self.needUpdate:
			for b in self.basketsList:
				d = os.stat(b.path).st_mtime
				if d!=b.timestamp:
					self.needUpdate=True
					b.timestamp=d
			if not self.needUpdate:
				return self.notesList
		self.getNotes()
		#print self.notesList
		self.readNotes()
		return self.notesList

	# gets baskets list
	def getBaskets(self):
		stat = os.stat(self.base+"baskets/baskets.xml")
		if self.lastBasketListChange==stat.st_mtime:
			print 'Basket list didn\'t change'
			return
		self.needUpdate = True
		self.basketsList=[]
		self.lastBasketListChange = stat.st_mtime
		print 'Looking for baskets'
		doc = parse (self.base+"baskets/baskets.xml")
		baskets = doc.getElementsByTagName("basket")
		
		for b in baskets :
			folder = b.attributes.get("folderName", None)
			prot = b.getElementsByTagName('protection')[0]
			if prot.attributes.get('type').value != '0': # basket is protected
				continue # skip it
			name = b.getElementsByTagName('name')[0]
			name=name.childNodes[0].nodeValue
			#print name + '==================================='
			btmp = Basket(self.base+'baskets/'+folder.value, name)
			self.basketsList.append( btmp)
			btmp.timestamp = 0;
		print len(self.basketsList), 'baskets found'

	# extracts notes from the baskets
	def getNotes(self):
		print 'Looking for notes'
		for B in self.basketsList:
			b=B.path
			print 'in',b
			try:
				doc = parse ( b + u'.basket')
			except:
				print "Error while parsing basket", b
			notes = doc.getElementsByTagName("note")

			if len(self.desiredTag)==1:
				hideDesiredTag=True
			else:
				hideDesiredTag=False
			for n in notes:
				typeN = n.attributes.get("type", None)
				if typeN == None:
					continue
				typeN = typeN.value
				
				if typeN != u'html' and typeN != u'image':# and typeN != u'link':	# only show text or link notes
					continue
				tag = n.getElementsByTagName("tags")
				if len(tag)==0:
					continue
				tags = tag[0].childNodes[0].nodeValue
				tags = tags.rsplit(';')
				skipNote = True
				for seltag in self.desiredTag:
					if  seltag in tags:	# skip the note if there isn't the "desktop" tag
						skipNote = False
						break
				if skipNote:
					continue
				if hideDesiredTag:
					print 'Removing tag', self.desiredTag[0]
					tags.remove(self.desiredTag[0])
				ct = n.getElementsByTagName("content")[0]
				c =ct.childNodes[0]
				#switch according to note's type
				if typeN == u'html': # text
					newNote = Note(b+c.nodeValue, 'text', B)
				#elif typeN == u'link':	# link
					#linkText = ct.attributes.get("title")
					#linkText = linkText.childNodes[0].nodeValue
					#newNote = Note(None)
				else: # image
					newNote = Note(b+c.nodeValue, 'img', B)
					newNote.text = '<img src="'+b+c.nodeValue+'" height="54" />'
				if myConfig.showTags:
					for t in tags:	# save the note's tags
						#print 'tag', t, self.tags.has_key(t)
						if self.tags.has_key(t):
							newNote.otherTags.append(self.tags[t])
				self.notesList.append(newNote)
			print len(self.notesList),"notes found"

	# reads notes contents
	def readNotes(self):
		try:
			hlFile = open (myConfig.hlFilePath, 'rb')
			highlights = marshal.load(hlFile)
			hlFile.close()
		except:
			highlights = {}
		highlightsNew = {}
		print 'Reading notes content'
		text = ''
		for n in self.notesList:
			#print "Reading", n
			if n.path==None or not os.path.isfile(n.path) :
				continue
			if n.type == 'text':
				
				doc = open (n.path)
				data = doc.read()
				doc.close()
				doc = open (n.path)
				self.myHTMLParser.reset()
				self.myHTMLParser.feed(data)
				self.myHTMLParser.close()
				c = self.myHTMLParser.getPositions()
				aa = self.readPartOfFile(doc, c[0], c[1])
				n.text = aa
				try:
					n.text = unicode(aa, 'utf8')
				except:
					print "Error while decoding note(utf8)"
					try:
						n.text = unicode(aa, 'Latin1')
					except:
						print "Error while decoding note(Latin1)"
				doc.close()
			try:
				tmp = highlights[n.path]
				highlightsNew[n.path]=tmp
				n.isHighlighted=tmp
			except:
				pass
			#print n, "read"
		try:
			hlFile = open (myConfig.hlFilePath, 'wb')
			marshal.dump(highlightsNew, hlFile)
			hlFile.close()
		except:
			pass
		print 'ok, all notes read'

	# reads file from start position to end position
	def readPartOfFile(self, doc, start, end):
		#print start, end
		for i in range(start[0]-1):
			doc.readline()
		text = ''
		if start[0] < end[0]-1:
			for i in range(start[0], end[0]):
				text = text + doc.readline()
		lastLine = doc.readline()
		lastLine = lastLine[:end[1]]
		text = text + lastLine
		return text[start[1]:]

	# reloads list of tags 
	def reloadTags(self):
		if not os.path.isfile(self.base+ 'tags.xml'):
			print "Tags file not found !"
			self.tags={}
			return
		stat = os.stat(self.base+ 'tags.xml')
		if stat.st_mtime==self.lastTagsChange:
			print 'Tags didn\'t change'
			return
		self.needUpdate = True
		self.lastTagsChange=stat.st_mtime
		print 'Loading tags'
		self.tags = {}
		doc = parse (self.base+ 'tags.xml')
		#print 'Ok, tags file parsed'
		tags = doc.getElementsByTagName('tag')
		#print "Found", len(tags), "tags"
		for t in tags:
			states = t.getElementsByTagName("state")
			#print "Found", len(states), "states"
			for st in states:
				tagId = st.attributes.get('id').value
				name=[]
				if len(states)==1:
					name = t.getElementsByTagName("name")[0].childNodes
				else:
					try:
						name = st.getElementsByTagName("name")[0].childNodes
					except:
						pass
				if name==[]:
					name = tagId
				else:
					name = name[0].nodeValue
				self.choosingTags[name]=tagId
				#print name, '=====>', tagId
				icon = st.getElementsByTagName("emblem")
				if icon==[]:
					continue
				icon = icon[0].childNodes
				if len(icon) != 0:
					iconName = icon[0].nodeValue
					#if icon[0].nodeValue.startswith('/'):
					self.tags[tagId]= str(dispElmt.iconLoader.iconPath(iconName, KIcon.Small, False))
		#print self.tags

# initialize locale
def init_language(widget):

	global _
	lang = karamba.userLanguages(widget)
	langFile = None

	for l in lang:
		fileContents = karamba.readThemeFile(widget, 'locales/'+l+'.mo')
		if fileContents.__len__() > 0:
			gt = GNUTranslations(StringIO(fileContents))
			_ = gt.ugettext
			break

#this is called when your widget is initialized
def initWidget(widget):
	
	global _, notesGatherer, myConfig, myPostIts, dispElmt, myActions
	print "Starting init..."

	myConfig = Desktopbasket_config(widget)

	init_language(widget)
	dispElmt = DisplayElements(widget, myConfig, _)

	karamba.addMenuConfigOption(widget, 'showTags', _('Show tags'))
	karamba.setMenuConfigOption(widget, 'showTags', myConfig.showTags)
	karamba.addMenuConfigOption(widget, 'titleDate', _('Show clock in title'))
	karamba.setMenuConfigOption(widget, 'titleDate',myConfig.titleType=='Date')
	
	notesGatherer = NotesGatherer()
	myActions = Desktopbasket_actions()

	if not myConfig.desiredTag:
		ans = os.spawnlp(os.P_WAIT, 'kdialog', 'kdialog', '--yesno', _("Do you want to create and use the \"Desktop\" tag\nto display Basket's notes ?"), '--caption', 'DesktopBasket')
		if ans :
			myConfig.desiredTag=[]
			askForDesiredTag(widget)
		else:
			createDesktopTag(widget, False)

	myPostIts = []
	tmpSmall = myConfig.smallNote
	myConfig.smallNote = False
	reloadNotes(widget)
	myConfig.smallNote = tmpSmall
	if myConfig.smallNote:
		karamba.resizeWidget(widget, myConfig.size[0], 20)
		print "Init done"
	karamba.redrawWidget(widget)

#this is called everytime your widget is updated
#the update inverval is specified in the .theme file
def widgetUpdated(widget):

	global cmpt_refresh

	if myConfig.titleType=='Date':
		date = time.localtime()
		myConfig.titleText = time.strftime('%X', date)+'  '+ time.strftime('%x', date)
		karamba.changeText(widget, dispElmt.title, myConfig.titleText+" ("+ str(currentPostIt+1)+'/'+str(len(myPostIts))+')')

	if cmpt_refresh>0:
		cmpt_refresh = cmpt_refresh-1
	else:
		cmpt_refresh = myConfig.refreshRate
		reloadNotes(widget)
	karamba.redrawWidget(widget)

def reloadNotes(widget):
	global notesGatherer, skippedOnce, currentPostIt, myPostIts, myConfig
	
	if myConfig.smallNote:
		skippedOnce = True
		return
	skippedOnce = False
	
	notesTmp = notesGatherer.main()

	if not notesGatherer.needUpdate:
		print "Notes didn\'t change since last update"
		return

	print "Notes changed recently, update needed"
	for pi in myPostIts:
		pi.delete(widget)
		
	myPostIts = []
	myNotes = notesTmp
	vpos = 20
	p = PostIt(myConfig)
	myPostIts.append(p)
	prev_basket=None
	for n in myNotes:
		if myConfig.basketNameColor!='Don\'t show' and n.basket != prev_basket:
			prev_basket=n.basket
			kbn = karambaBasketName(widget, n, vpos, myConfig)
			vpos = vpos + karamba.getRichTextSize(widget, kbn.rt)[1]
		else:
			kbn=None
		kn = karambaNote(widget, n, vpos, myConfig, dispElmt.highlightIcon)
		vpos = vpos + karamba.getRichTextSize(widget, kn.rt)[1]+3
		if vpos > myConfig.size[1]-myConfig.yborder: 	# end of note is out of the post-it
			if p.notes!=[]:
				p = PostIt(myConfig)	# create a new Post-It
				vpos = 20
				if myConfig.basketNameColor!='Don\'t show':
					if kbn:
						kbn.move(widget, vpos)
					else:
						kbn = karambaBasketName(widget, n, vpos, myConfig)
					vpos = vpos + karamba.getRichTextSize(widget, kbn.rt)[1]+3
					p.append(kbn)
				kn.move(widget, vpos)
				vpos = vpos + karamba.getRichTextSize(widget, kn.rt)[1]+3
				myPostIts.append(p)	# and add it to the post-it list
			p.append(kn)
			if vpos > myConfig.size[1]-myConfig.yborder:
				# the note is too large to be displayed entirely
				p.setExtendable(widget)
		else:
			if kbn:
				p.append(kbn)
			p.append(kn)
	if len(myPostIts)<=0:
		currentPostIt = 0
	elif len(myPostIts)<=currentPostIt:
		currentPostIt = len(myPostIts)-1
	updateCurrentPostIt(widget)
	notesGatherer.needUpdate = False
	
def updateCurrentPostIt(widget):
	global currentPostIt, myPostIts
	pi_index = 0
	for pi in myPostIts:
		pi.show(widget, pi_index==currentPostIt)
		pi_index = pi_index+1
	karamba.changeText(widget, dispElmt.title, myConfig.titleText+" ("+ str(currentPostIt+1)+'/'+str(len(myPostIts))+')')
	karamba.changeTextColor(widget, dispElmt.title, int((Desktopbasket_config.colors[myConfig.defaultColor])[0:3]), int((Desktopbasket_config.colors[myConfig.defaultColor])[3:6]), int((Desktopbasket_config.colors[myConfig.defaultColor])[6:9]))


#This gets called everytime our widget is clicked.
#Notes:
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
#                    4,5 = Scroll wheel up and down
def widgetClicked(widget, x, y, button):
    pass

#This gets called everytime our widget is clicked.
#Notes
#  widget = reference to our widget
#  x = x position (relative to our widget)
#  y = y position (relative to our widget)
#  botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    #Warning:  Don't do anything too intensive here
    #You don't want to run some complex piece of code everytime the mouse moves
    pass

#This gets called when an item is clicked in a popup menu you have created.
#  menu = a reference to the menu
#  id = the number of the item that was clicked.
def menuItemClicked(widget, menu, id):

	global myActions, dispElmt, myConfig, _
	# change title
	if id==dispElmt.configOptions['title']:
		inputtext = karamba.readConfigEntry(widget, 'title')
		command = ["kdialog", '--caption', 'DesktopBasket',"--inputbox", _('Enter new title'), inputtext]
		myActions.select_title_pid = karamba.executeInteractive(widget, command)
	# select text default colour
	elif id==dispElmt.configOptions['color']:
		command = ["kdialog", '--caption', 'DesktopBasket',"--combobox", _('Select text default color')]
		cTrans=[]
		for c in Desktopbasket_config.colors.keys():
			cTrans.append( _(c) )
		cTrans.sort()
		command.extend(cTrans)
		myActions.select_color_pid = karamba.executeInteractive(widget, command)
	# change size
	elif id==dispElmt.configOptions['size']:
		s = str(myConfig.size[0])+' x '+str(myConfig.size[1])
		command = ["kdialog", '--caption', 'DesktopBasket',"--inputbox", _('Enter new size'), s]
		myActions.select_size_pid = karamba.executeInteractive(widget, command)
	#change background
	elif id==dispElmt.configOptions['background']:
		pth = myConfig.imagesPath
		command = ["kdialog", '--caption', 'DesktopBasket',"--getopenfilename", pth]
		myActions.select_img_pid = karamba.executeInteractive(widget, command)
	# change title bar background
	elif id==dispElmt.configOptions['titleBar']:
		pth = myConfig.imagesPath
		command = ["kdialog", '--caption', 'DesktopBasket', "--getopenfilename", pth]
		myActions.select_ttBar_pid = karamba.executeInteractive(widget, command)
	# choose selecting tag
	elif id==dispElmt.configOptions['selecttag']:
		askForDesiredTag(widget)
	# use default backgrounds
	elif id==dispElmt.configOptions['resetTheme']:
		myConfig.bgImgFile = myConfig.imagesPath+'/background.png'
		karamba.deleteImage(widget, dispElmt.bgImg)
		dispElmt.bgImg = karamba.createBackgroundImage(widget, 0, 20, myConfig.bgImgFile)
		karamba.resizeImage(widget, dispElmt.bgImg, myConfig.size[0], myConfig.size[1]-20)
		karamba.writeConfigEntry(widget, "bgImg", myConfig.bgImgFile)
		myConfig.ttBarImgFile = myConfig.imagesPath+'/TitleBar.png'
		karamba.deleteImage(widget, dispElmt.ttBarImg)
		dispElmt.ttBarImg = karamba.createBackgroundImage(widget, 0, 0, myConfig.ttBarImgFile)
		karamba.resizeImage(widget, dispElmt.ttBarImg, myConfig.size[0], 20)
		karamba.writeConfigEntry(widget, "titleBar", myConfig.ttBarImgFile)
		karamba.redrawWidget(widget)
	# force refresh
	elif id==dispElmt.configOptions['refresh']:
		notesGatherer.needUpdate=True
		reloadNotes(widget)
		karamba.redrawWidget(widget)
	# choose refresh rate
	elif id==dispElmt.configOptions['refreshRate']:
		command = ["kdialog", '--caption', 'DesktopBasket',"--inputbox", _('Enter new refresh rate (s)'), str(myConfig.refreshRate)]
		myActions.select_refresh_pid = karamba.executeInteractive(widget, command)
	# choose baskets names colour
	elif id==dispElmt.configOptions['basketNameColor']:
		command = ["kdialog", '--caption', 'DesktopBasket',"--combobox", _('Select basket name color')]
		cTrans=[_('Don\'t show')]
		for c in Desktopbasket_config.colors.keys():
			cTrans.append( _(c) )
		cTrans.sort()
		command.extend(cTrans)
		myActions.select_basketnamecolor_pid = karamba.executeInteractive(widget, command)
	# show about dialog
	elif id==dispElmt.configOptions['about']:
		aboutString = '\"DesktopBasket '+Desktopbasket_config.version+'\n'+_('Created by')+' :\n'+\
		Desktopbasket_config.authors+'\n\n'+_('Contributors')+' :\n'+\
		Desktopbasket_config.contributors+'\"'
		command = 'kdialog --caption DesktopBasket --msgbox '+aboutString+' --title \"'+ _('About')+'\"'
		karamba.execute(command)
	#create desktop tag
	elif id==dispElmt.configOptions['createDesktopTag']:
		createDesktopTag(widget, True)
		
	# choose highlight colour
	else:
		for color in dispElmt.highlightOptions.keys():
			if id==dispElmt.highlightOptions[color]:
				hlCurrentNote.changeHighlight(widget, color)
				break

# creates the desktop tag if it doesn't exist and adds it to the selecting tags
def createDesktopTag(widget, askConfirm):
	global myConfig, _
	tagsFile = open(myConfig.basketBase+'tags.xml', 'r')
	content = tagsFile.read()
	tagsFile.close()
	if re.search('<state id=\"desktop\"',content):
		text = _('The \"Desktop\" tag already exists')
		karamba.execute('kdialog --msgbox \''+ text + '\' --caption DesktopBasket')
	else:
		if askConfirm:
			ans = os.spawnlp( os.P_WAIT, 'kdialog', 'kdialog', '--yesno', _('Do you really want to create \nthe \"Desktop\" tag in Basket ?'))
		else:
			ans=0
		if ans==0:
			dtag = '\n  <tag>\n  <name>'+str(_('Desktop'))+'</name>\n  <shortcut></shortcut>\n  <inherited>false</inherited>\n  <state id="desktop" >\n   <name>'+str(_('Desktop'))+'</name>\n   <emblem>desktop</emblem>\n   <text strikeOut="false" bold="false" underline="false" color="" italic="false" />\n   <font size="-1" name="" />\n   <backgroundColor></backgroundColor>\n   <textEquivalent string="" onAllTextLines="false" />\n  </state>\n </tag>\n</basketTags>'
			newcontent = content.replace('</basketTags>', dtag)
			tagsFile = open(myConfig.basketBase+'/tags.xml', 'w')
			tagsFile.write(newcontent)
			tagsFile.close()
			if getoutput('dcop | grep basket'):
				hidden = getoutput('dcop basket MainWindow hidden')
				ans = os.spawnlp(os.P_WAIT, 'kdialog', 'kdialog', '--yesno', _('Basket needs to be restarted.\nDo you want to restart it now ?'), '--caption', 'DesktopBasket')
				if ans==0:
					os.spawnlp(os.P_WAIT, 'dcop', 'dcop', 'basket', 'MainApplication-Interface', 'quit')
					os.spawnlp(os.P_WAIT, 'basket', 'basket')
					if hidden=='true':
						karamba.execute('dcop basket MainWindow hide')
	if myConfig.desiredTag:
		if 'desktop' not in myConfig.desiredTag:	# if 'desktop' is already a selecting tag, don't append it to the list
			myConfig.desiredTag.append('desktop')
	else:
		myConfig.desiredTag = ['desktop']
	myConfig.saveDesiredTag(widget)
			
def askForDesiredTag(widget):
	global myActions, myConfig, notesGatherer, _
	command = ['kdialog', '--caption', 'DesktopBasket', '--checklist', _('Select tags')]
	a = notesGatherer.choosingTags.keys()
	a.sort()
	for i in a:	# create list for kdialog
		command.append(notesGatherer.choosingTags[i])	# kdialog tag
		command.append(i)	# kdialog item
		if notesGatherer.choosingTags[i] in myConfig.desiredTag:
			command.append('on')	# select item in kdialog
		else:
			command.append('off')
	myActions.select_tag_pid = karamba.executeInteractive(widget, command)

#This gets called when an item is clicked in the theme CONFIGURATION menu,
#not the popup menus that you create.
#  key = the reference to the configuration key that was changed
#  value = the new value (true or false) that was selected
def menuOptionChanged(widget, key, value):
	global notesGatherer, myConfig, dispElmt
	if key=='showTags':
		myConfig.showTags= value==1
		notesGatherer.needUpdate=True
		karamba.writeConfigEntry(widget, "showTags", str(myConfig.showTags))
	elif key=='titleDate':
		if value==1:
			myConfig.titleType= 'Date'
		else:
			myConfig.titleType= 'Text'
			myConfig.titleText = karamba.readConfigEntry(widget, "title")
			if not myConfig.titleText:
				myConfig.titleText="DesktopBasket"
			updateCurrentPostIt(widget)
		karamba.writeConfigEntry(widget, "titleType", myConfig.titleType)

#This gets called when a meter (image, text, etc) is clicked.
# NOTE you must use attachClickArea() to make a meter
# clickable.  
#  widget = reference to your theme
#  meter = the meter clicked
#  button = the button clicked (see widgetClicked for button numbers)
def meterClicked(widget, meter, button):
	global myPostIts, currentPostIt, skippedOnce
	global hlCurrentNote, myConfig
	if button==1:	#left button
		if meter == dispElmt.moveLeftButton or meter == dispElmt.moveRightButton:
			if myConfig.smallNote:
				return
			if meter == dispElmt.moveLeftButton and currentPostIt > 0 :
				currentPostIt = currentPostIt-1
			elif meter == dispElmt.moveRightButton and currentPostIt < len(myPostIts)-1 :
				currentPostIt = currentPostIt+1
			updateCurrentPostIt(widget)
		elif meter == dispElmt.launchBasketButton:
			print 'Running basket'
			if getoutput('dcop | grep basket'):
				karamba.execute('dcop basket MainWindow restore')
			else:
				karamba.execute('basket')
		elif meter == dispElmt.title:
			myConfig.smallNote = not myConfig.smallNote
			karamba.writeConfigEntry(widget, "isReduced", str(myConfig.smallNote))
			if myConfig.smallNote:
				karamba.resizeWidget(widget, myConfig.size[0], 20)
			else:
				if skippedOnce:
					print "Reloading because the note was minimized for too long"
					reloadNotes(widget)
					karamba.redrawWidget(widget)
				karamba.resizeWidget(widget, myConfig.size[0], myConfig.size[1])
		elif meter == dispElmt.cfgButton:
			karamba.popupMenu(widget, dispElmt.configMenu, myConfig.size[0]-100-myConfig.xborder, 20 )
		elif meter == myPostIts[currentPostIt].extendButton:
			myPostIts[currentPostIt].extendNote(widget, 5000, " ("+ str(currentPostIt+1)+'/'+str(len(myPostIts))+')')
		else:
			for kn in myPostIts[currentPostIt].notes:
				if meter == kn.hlButton:
					pos = karamba.getImagePos(widget, kn.hlButton)
					hlCurrentNote = kn
					karamba.popupMenu(widget, dispElmt.highlightMenu, pos[0], pos[1]+18)
	elif button==2:		#middle clic
		if meter == myPostIts[currentPostIt].extendButton:
			myPostIts[currentPostIt].extendNote(widget, 0, " ("+ str(currentPostIt+1)+'/'+str(len(myPostIts))+')')

#This gets called when a command you have executed with executeInteractive() outputs something
#to stdout.  This way you can get the output of for example kdialog without freezing up the widget
#waiting for kdialog to end.
#  widget = reference to your theme
#  pid = process number of the program outputting (use this if you execute more than out process)
#  output = the text the program outputted to stdout
def commandOutput(widget, pid, output):
	
	global myConfig, notesGatherer, cmpt_refresh, myActions, dispElmt

	if output =='\n':
		return
	
	if pid == myActions.select_img_pid:
		myConfig.bgImgFile = output
		karamba.deleteImage(widget, dispElmt.bgImg)
		dispElmt.bgImg = karamba.createBackgroundImage(widget, 0, 20, myConfig.bgImgFile)
		karamba.resizeImage(widget, dispElmt.bgImg, myConfig.size[0], myConfig.size[1]-20)
		karamba.writeConfigEntry(widget, "bgImg", myConfig.bgImgFile)
		karamba.redrawWidget(widget)

	elif pid == myActions.select_ttBar_pid:
		myConfig.ttBarImgFile = output
		karamba.deleteImage(widget, dispElmt.ttBarImg)
		dispElmt.ttBarImg = karamba.createBackgroundImage(widget, 0, 0, myConfig.ttBarImgFile)
		karamba.resizeImage(widget, dispElmt.ttBarImg, myConfig.size[0], 20)
		karamba.writeConfigEntry(widget, "titleBar", myConfig.ttBarImgFile)
		karamba.redrawWidget(widget)

	# change title
	elif pid == myActions.select_title_pid:
		myConfig.titleText = output[:-1]
		karamba.writeConfigEntry(widget, "title", myConfig.titleText)
		try:
			myConfig.titleText = unicode(myConfig.titleText, 'utf-8')
		except:
			try:
				myConfig.titleText = unicode(myConfig.titleText, 'Latin1')
			except:
				pass
		karamba.changeText(widget, dispElmt.title, myConfig.titleText+" ("+ str(currentPostIt+1)+'/'+str(len(myPostIts))+')')
		myConfig.titleType= 'Text'
		karamba.writeConfigEntry(widget, "titleType", myConfig.titleType)
		karamba.setMenuConfigOption(widget, 'titleDate', False)
		karamba.redrawWidget(widget)

	# change text default color
	elif pid == myActions.select_color_pid:
		myConfig.defaultColor = output[:-1]
		for c in Desktopbasket_config.colors.keys():
			if myConfig.defaultColor==_(c):
				myConfig.defaultColor=c
				break
		karamba.writeConfigEntry(widget, "defaultColor", myConfig.defaultColor)
		tmpSmall = myConfig.smallNote
		myConfig.smallNote = False
		notesGatherer.needUpdate=True
		reloadNotes(widget)
		karamba.redrawWidget(widget)
		myConfig.smallNote = tmpSmall

	# change size
	elif pid == myActions.select_size_pid:
		s = output[:-1].lower()
		if s.find('x') == -1:
			return
		ss = s.split('x')
		myConfig.size[0]= int(ss[0])
		myConfig.size[1]= int(ss[1])
		karamba.resizeWidget(widget, myConfig.size[0], myConfig.size[1])
		karamba.resizeImage(widget, dispElmt.bgImg, myConfig.size[0], myConfig.size[1]-20)
		karamba.resizeImage(widget, dispElmt.ttBarImg, myConfig.size[0], 20)
		karamba.moveImage(widget, dispElmt.launchBasketButton, myConfig.size[0]-16-myConfig.xborder, 2)
		karamba.moveImage(widget, dispElmt.cfgButton, myConfig.size[0]-36-myConfig.xborder, 2)
		karamba.resizeText(widget, dispElmt.title, myConfig.size[0]-100-2*myConfig.xborder, 20)
		karamba.writeConfigEntry(widget, "noteHeight", str(myConfig.size[1]))
		notesGatherer.needUpdate=True
		if myConfig.smallNote:
			karamba.resizeWidget(widget, myConfig.size[0], 20)
		notesGatherer.needUpdate=True
		reloadNotes(widget)
		karamba.redrawWidget(widget)

	# change selecting tag
	elif pid == myActions.select_tag_pid:
		tmp = output[:-2].rsplit(' ')	# values are separated with spaces
		myConfig.desiredTag=[]
		for i in tmp:
			myConfig.desiredTag.append(i[1:-1])
		print "New preferred tags are", myConfig.desiredTag
		myConfig.saveDesiredTag(widget)
		cmpt_refresh = 1
		karamba.redrawWidget(widget)

	# change baskets names colour
	elif pid == myActions.select_basketnamecolor_pid:
		color = output[:-1]
		if color==_('Don\'t show'):
			color='Don\'t show'
		else:
			for c in Desktopbasket_config.colors.keys():
				if color==_(c):
					color=c
					break
		myConfig.basketNameColor = color
		karamba.writeConfigEntry(widget, "basketNameColor", color)
		tmpSmall = myConfig.smallNote
		myConfig.smallNote = False
		notesGatherer.needUpdate=True
		reloadNotes(widget)
		karamba.redrawWidget(widget)
		myConfig.smallNote = tmpSmall

	# change refresh rate
	elif pid == myActions.select_refresh_pid:
		myConfig.refreshRate = int(output[:-1].lower())
		karamba.writeConfigEntry(widget, "myConfig.refreshRate", str(myConfig.refreshRate))

#This gets called when another theme sends a message.
#  widget = reference to your theme
#  caller = "pretty" name or regular name (name of theme file with a number attached) of sending theme
#  output = the text sent.
def themeNotify(widget, caller, message):
    pass

#This gets called when an item is dropped on this widget.
# NOTE you have to call acceptDrops() before your widget will accept drops.
#  widget = reference to your theme
#  dropText = the text of the dropped item (probably a URL to it's location in KDE)
#  x = the x position on the theme relative to the upper left corner
#  y = the y position on the theme relative to the upper left corner
def itemDropped(widget, dropText, x, y):
    pass


#This gets called when a new program is LOADING in KDE.  When it is done
#loading, startupRemoved() is called, followed by taskAdded().
#  widget = reference to your widget
#  task = A refence to the task that is starting.  
def startupAdded(widget, startup):
    pass

#This gets called when a new program is done LOADING in KDE.
#  widget = reference to your widget
#  task = A refence to the task that just finished loading.  
def startupRemoved(widget, startup):
    pass

#This is called every time a new task (program) is started in KDE.
#  widget = reference to your widget
#  task = A refence to the new task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def taskAdded(widget, task):
    pass

#This is called everytime a task (program) is closed in KDE.
#  widget = reference to your widget
#  task = A refence to the task.  
def taskRemoved(widget, task):
    pass

#This is called everytime a different task gains focus (IE, the user clicks
#on a different window).  
#  widget = reference to your widget
#  task = A refence to the task.  Call getTaskInfo() with this reference
#         to get the name, etc of this new task.
def activeTaskChanged(widget, task):
    pass

#This is called everytime the systray you created with createSystray() is updated
def systrayUpdated(widget):
    pass

#This is called everytime the current desktop changes
#  widget = reference to your widget
#  desktop = the current desktop
def desktopChanged(widget, desktop):
    pass

#This is called everytime the wallpaper changes on a desktop 
#  widget = reference to your widget
#  desktop = the desktop whose wallpaper changed
def wallpaperChanged(widget, desktop):
    pass

#This is called everytime there is a key press in any focused input field
#  widget = reference to your widget
#  meter = reference to an input box
#  char = the key that was pressed
def keyPressed(widget, meter, char):
    pass

def widgetClosed(widget):
    pass

# This will be printed when the widget loads.
print "Loaded DesktopBasket"
