#!/usr/bin/python
# -*- coding: utf-8 -*-
############################################################################
#    Copyright (C) 2007 by Romain Henriet                                  #
#    henriet.romain@libertysurf.fr                                         #
#                                                                          #
#    DesktopBasket is free software; you can redistribute it and/or modify #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    DesktopBasket is distributed in the hope that it will be useful,      #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

from kdecore import *
from commands import getoutput
import os, re
import karamba

class Desktopbasket_config:

	#########################################################################
	#                       Theme's version                                 #
	#                    should be updated for each release                 #
	#########################################################################
	version = "1.4"

	authors = 'Romain Henriet (henriet.romain@libertysurf.fr)'
	contributors = 'Suslik (dd@accentsolution.com)'
	
	# HTML colors
	colors = {'black':'000000000', 'white':'255255255', 'red':'255000000', 'blue':'000000255', 'yellow':'2552550', 'orange':'255165000', 'green':'000255000', 'purple':'160032240', 'maroon':'176048096', 'grey':'190190190', 'navy':'000000128', 'silver': '230230250'}

	def __init__(self, widget):

		print "Loading config"
		self.basketBase = self.searchForBasketBase()

		self.xborder=20
		self.yborder=12

		self.size=[0,0]
		
		self.smallNote = karamba.readConfigEntry(widget, "isReduced")=='True'

		self.showTags = karamba.readConfigEntry(widget, "showTags")!='False'
		
		self.desiredTag = karamba.readConfigEntry(widget, "desiredTag")
		if self.desiredTag:
			self.desiredTag=self.desiredTag.rsplit(';')

		self.titleType = karamba.readConfigEntry(widget, "titleType")
		if not self.titleType:
			self.titleType='Text'
			karamba.writeConfigEntry(widget, "titleType", self.titleType)
		if self.titleType=='Date':
			self.titleText = ""
		else:
			self.titleText = karamba.readConfigEntry(widget, "title")
			if not self.titleText:
				self.titleText = 'DesktopBasket'
				karamba.writeConfigEntry(widget, "title", self.titleText)
		
		self.size[0] = karamba.readConfigEntry(widget, "widgetWidth")
		if not self.size[0]:
			self.size[0]=350

		self.refreshRate = karamba.readConfigEntry(widget, "refreshRate")
		if not self.refreshRate:
			self.refreshRate=30

		self.basketNameColor = karamba.readConfigEntry(widget, "basketNameColor")
		if not self.basketNameColor:
			self.basketNameColor='Don\'t show'

		self.size[1] = karamba.readConfigEntry(widget, "noteHeight")
		if not self.size[1]:
			self.size[1]=200
			karamba.writeConfigEntry(widget, "noteHeight", str(self.size[1]))
			
		self.defaultColor = karamba.readConfigEntry(widget, "defaultColor")
		if not self.defaultColor:
			self.defaultColor = 'black'
			karamba.writeConfigEntry(widget, "defaultColor", self.defaultColor)
		else:
			try:
				self.titleText=unicode(self.titleText, 'utf8')
			except:
				try:
					self.titleText=unicode(self.titleText, 'Latin1')
				except:
					pass

		try:
			# i think open(blah).readline() closes the file auto, just like .read()
			# python is generally good at garbadge collection.
			self.installedVersion = open(os.environ['HOME']+'/.superkaramba/desktopbasket/images/version').readline()
			print self.installedVersion
		except:
			self.installedVersion = '0'

		themePath = karamba.getThemePath(widget)

		self.hlFilePath = os.environ['HOME']+'/.superkaramba/desktopbasket/highlights'

		self.imagesPath = os.environ['HOME']+'/.superkaramba/desktopbasket'
		if self.installedVersion != Desktopbasket_config.version:
			if not os.path.exists(self.imagesPath):
				os.makedirs(self.imagesPath)
			if not themePath.endswith(".skz"):
				os.spawnlp(os.P_WAIT, 'cp', 'cp', '-r', themePath+'/images', self.imagesPath)
				self.imagesPath = self.imagesPath+'/images'
			else:
				# extract the folder
				print '\"Images\" folder will be extracted'
				os.spawnlp(os.P_WAIT, 'unzip', 'unzip', '-o', themePath, 'images/*', '-d', self.imagesPath)
				self.imagesPath = self.imagesPath+'/images'
			f=open(self.imagesPath+'/version', 'w')
			f.write(Desktopbasket_config.version)
			f.close()
			# move old version's highlight file if there is one
			oldHlFilePath = os.environ['HOME']+'/.superkaramba/dektopbasket_highlights'
			if os.path.exists(oldHlFilePath):
				os.spawnlp(os.P_WAIT, 'mv', 'mv', oldHlFilePath, self.hlFilePath)
		else:
			self.imagesPath = self.imagesPath+'/images'

		self.installedVersion = Desktopbasket_config.version

		self.bgImgFile = karamba.readConfigEntry(widget, "bgImg")
		if not self.bgImgFile:
			self.bgImgFile = self.imagesPath+'/background.png'
			karamba.writeConfigEntry(widget, "bgImg", self.bgImgFile)
			
		self.ttBarImgFile = karamba.readConfigEntry(widget, "titleBar")
		if not self.ttBarImgFile:
			self.ttBarImgFile = self.imagesPath+'/TitleBar.png'
			karamba.writeConfigEntry(widget, "titleBar", self.ttBarImgFile)
		
		print "Config Loaded"

	def searchForBasketBase(self):
		# a small rewrite. This takes care of more potential errors
		basketBase = ''
		a = getoutput('kde-config --localprefix')
		if a and not a.count('kde-config'): # if there was an error of any sort, that string will be in the responce.
			try:
				f = open(a+'share/config/basketrc').read() # this way you don't need to close it. does it by itself after read
				basketBase = re.search('(?<=dataFolder=).+',f).group().replace('$HOME',os.environ['HOME']) + '/'
				# this is a bit long, but if any part above fails, we don't need anyof this.
				# thus, I put it all under one try
			except:
				# I bet default basket install puts something there... so, something better than "return" with empty hands
				basketBase = a + 'share/apps/basket/'
				# we can also make it a '' and check for it later.
			print 'Basket baskets folder :', basketBase
		# i hope you do not mind my switching to return basketBase.
		# it makes it easier to reuse this function later somewhere
		return basketBase

	def saveDesiredTag(self, widget):
		# writes the list of tags that select notes to display into the config file
		selString = ''
		for t in self.desiredTag:
			selString = selString+t+';'		# separates tags with ';'
		selString = selString[:-1]
		karamba.writeConfigEntry(widget, "desiredTag", selString)
	
class DisplayElements:

	def __init__(self, widget, config, _):

		self.widget= widget
		self.config = config
		
		self.iconLoader = KGlobal.iconLoader()

		karamba.resizeWidget(widget, config.size[0], config.size[1])
		self.bgImg = karamba.createBackgroundImage(widget, 0, 20, config.bgImgFile)
		karamba.resizeImage(widget, self.bgImg, config.size[0],config.size[1]-20)
		
		self.ttBarImg = karamba.createBackgroundImage(widget, 0, 0, config.ttBarImgFile)
		karamba.resizeImage(widget, self.ttBarImg, config.size[0],20)
		
		self.moveLeftButton = karamba.createImage(widget, config.xborder, 2, str(self.iconLoader.iconPath('1leftarrow', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.moveLeftButton)
		
		self.moveRightButton = karamba.createImage(widget, 18+config.xborder, 2, str(self.iconLoader.iconPath('1rightarrow', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.moveRightButton)
		
		self.launchBasketButton = karamba.createImage(widget, config.size[0]-16-config.xborder, 2, str(self.iconLoader.iconPath('basket', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.launchBasketButton)
		
		self.cfgButton = karamba.createImage(widget, config.size[0]-36-config.xborder, 2, str(self.iconLoader.iconPath('configure', KIcon.Small, False)))
		karamba.attachClickArea(widget, self.cfgButton)
		
		self.title = karamba.createText(widget, config.xborder+40, -1, config.size[0]-80-2*config.xborder, 20, config.titleText)
		karamba.attachClickArea(widget, self.title)
		karamba.changeTextColor(widget, self.title, 0, 0, 0)
		karamba.setTextAlign(widget, self.title, 'CENTER')
		karamba.changeTextSize(widget, self.title, 14)

		self.configOptions = {}
		self.configMenu = karamba.createMenu(widget)
		self.configOptions['refresh'] = karamba.addMenuItem(widget, self.configMenu,_("Refresh"), 'reload')
		self.configOptions['refreshRate'] = karamba.addMenuItem(widget, self.configMenu,_("Change refresh rate"), 'reload')
		karamba.addMenuSeparator(widget, self.configMenu)
		self.configOptions['title'] = karamba.addMenuItem(widget, self.configMenu,_("Change title"), 'text')
		self.configOptions['color'] = karamba.addMenuItem(widget, self.configMenu,_("Change default text color"), 'colorize')
		self.configOptions['basketNameColor'] = karamba.addMenuItem(widget, self.configMenu,_("Change basket name color"), 'colorize')
		self.configOptions['size'] = karamba.addMenuItem(widget, self.configMenu,_("Change note size"), 'crop')
		self.configOptions['selecttag'] = karamba.addMenuItem(widget, self.configMenu,_("Choose tags"), 'find')
		self.configOptions['createDesktopTag'] = karamba.addMenuItem(widget, self.configMenu,_("Create and use the \"Desktop\" tag"), 'desktop')
		karamba.addMenuSeparator(widget, self.configMenu)
		self.configOptions['titleBar'] = karamba.addMenuItem(widget, self.configMenu,_("Change title bar"), 'frame_image')
		self.configOptions['background'] = karamba.addMenuItem(widget, self.configMenu,_("Change background"), 'frame_image')
		self.configOptions['resetTheme'] = karamba.addMenuItem(widget, self.configMenu,_("Reset theme"), 'remove')
		karamba.addMenuSeparator(widget, self.configMenu)
		self.configOptions['about'] = karamba.addMenuItem(widget, self.configMenu,_("About"), 'superkaramba')

		self.highlightOptions={}
		self.highlightMenu = karamba.createMenu(widget)
		a = os.listdir(config.imagesPath+'/highlight')
		self.highlightOptions["None"] = karamba.addMenuItem(widget, self.highlightMenu, '', config.imagesPath+'/highlight/None')
		for i in a:
			if i.endswith('.png'):
				self.highlightOptions[i] = karamba.addMenuItem(widget, self.highlightMenu, '', config.imagesPath+'/highlight/'+i)

		self.highlightIcon = str(self.iconLoader.iconPath('bookmark', KIcon.Small, False))


class Desktopbasket_actions:

	def __init__(self):
		self.select_img_pid = None
		self.select_title_pid = None
		self.select_size_pid = None
		self.select_color_pid = None
		self.select_ttBar_pid = None
		self.select_tag_pid = None
		self.select_refresh_pid = None
		self.select_basketnamecolor_pid = None
	